@@ -28,9 +28,10 @@ from google.api_core.exceptions import (
     InternalServerError,
     ServiceUnavailable,
 )
-from google.auth.exceptions import DefaultCredentialsError
+from google.auth.exceptions import DefaultCredentialsError, InvalidOperation
 from google.cloud import exceptions, storage
 
+from toil import memoize
 from toil.jobStores.abstractJobStore import (
     AbstractJobStore,
     JobStoreExistsException,
@@ -43,6 +44,7 @@ from toil.lib.compatibility import compat_bytes
 from toil.lib.io import AtomicFileCreate
 from toil.lib.misc import truncExpBackoff
 from toil.lib.retry import old_retry
+from toil.lib.url import URLAccess
 
 log = logging.getLogger(__name__)
 
@@ -116,7 +118,7 @@ def permission_error_reporter(url: ParseResult, notes: str) -> Iterator[None]:
     """
     try:
         yield
-    except exceptions.InvalidOperation as e:
+    except InvalidOperation as e:
         if "Anonymous credentials cannot be refreshed" in str(e):
             raise RuntimeError(
                 "Google Storage tried to refresh anonymous credentials. "
@@ -131,7 +133,7 @@ def permission_error_reporter(url: ParseResult, notes: str) -> Iterator[None]:
 
 
 
-class GoogleJobStore(AbstractJobStore):
+class GoogleJobStore(AbstractJobStore, URLAccess):
 
     nodeServiceAccountJson = "/root/service_account.json"
 
@@ -160,9 +162,10 @@ class GoogleJobStore(AbstractJobStore):
         self.storageClient, self.auth_notes = self.create_client()
 
     @classmethod
+    @memoize
     def create_client(cls) -> tuple[storage.Client, str]:
         """
-        Produce a client for Google Sotrage with the highest level of access we can get.
+        Produce a client for Google Storage with the highest level of access we can get.
 
         Fall back to anonymous access if no project is available, unlike the
         Google Storage module's behavior.
