@@ -6,14 +6,37 @@ import stat
 import sys
 import tempfile
 import uuid
-from collections.abc import Iterator
+from collections.abc import Iterator, Iterable
 from contextlib import contextmanager
 from io import BytesIO
 from typing import IO, Any, Callable, Optional, Protocol, Union
 
+from toil.lib.memoize import memoize
+from toil.lib.misc import StrPath
+
 logger = logging.getLogger(__name__)
 
 
+@memoize
+def get_toil_home() -> str:
+    """
+    Get the Toil home directory for storing configuration and global state.
+
+    Raises an error if it does not exist and cannot be created. Safe to run
+    simultaneously in multiple processes.
+    """
+   
+    # TODO: should this use an XDG config directory or ~/.config to not clutter the
+    # base home directory?
+    toil_home_dir = os.path.join(os.path.expanduser("~"), ".toil")
+
+    dir_path = try_path(toil_home_dir)
+    if dir_path is None:
+        raise RuntimeError(
+            f"Cannot create or access Toil configuration directory {toil_home_dir}"
+        )
+    return dir_path
+
 TOIL_URI_SCHEME = "toilfile:"
 
 
@@ -22,6 +45,9 @@ REMOTE_SCHEMES = STANDARD_SCHEMES + [TOIL_URI_SCHEME]
 ALL_SCHEMES = REMOTE_SCHEMES + ["file:"]
 
 def is_standard_url(filename: str) -> bool:
+    """
+    Return True if the given URL is a non-Toil, non-file: URL.
+    """
     return is_url_with_scheme(filename, STANDARD_SCHEMES)
 
 def is_remote_url(filename: str) -> bool:
@@ -49,16 +75,23 @@ def is_url_with_scheme(filename: str, schemes: list[str]) -> bool:
     return False
 
 def is_toil_url(filename: str) -> bool:
+    """
+    Return True if a URL is a toilfile: URL.
+    """
     return is_url_with_scheme(filename, [TOIL_URI_SCHEME])
 
 def is_file_url(filename: str) -> bool:
-    return is_url_with_scheme(filename, ["file:"])
+    """
+    Return True if a URL is a file: URL.
 
+    Will return False for bare paths.
+    """
+    return is_url_with_scheme(filename, ["file:"])
 
 def mkdtemp(
     suffix: Optional[str] = None,
     prefix: Optional[str] = None,
-    dir: Optional[str] = None,
+    dir: Optional[StrPath] = None,
 ) -> str:
     """
     Make a temporary directory like tempfile.mkdtemp, but with relaxed permissions.
@@ -153,31 +186,31 @@ def robust_rmtree(path: Union[str, bytes]) -> None:
             raise
 
 
-def atomic_tmp_file(final_path: str) -> str:
+def atomic_tmp_file(final_path: StrPath) -> str:
     """Return a tmp file name to use with atomic_install.  This will be in the
     same directory as final_path. The temporary file will have the same extension
     as finalPath.  It the final path is in /dev (/dev/null, /dev/stdout), it is
     returned unchanged and atomic_tmp_install will do nothing."""
     final_dir = os.path.dirname(os.path.normpath(final_path))  # can be empty
     if final_dir == "/dev":
-        return final_path
+        return str(final_path)
     final_basename = os.path.basename(final_path)
     final_ext = os.path.splitext(final_path)[1]
-    base_name = f"{final_basename}.{uuid.uuid4()}.tmp{final_ext}"
+    base_name = f"{final_basename}.{str(uuid.uuid4())}.tmp{final_ext}"
     return os.path.join(final_dir, base_name)
 
 
-def atomic_install(tmp_path, final_path) -> None:
+def atomic_install(tmp_path: StrPath, final_path: StrPath) -> None:
     """atomic install of tmp_path as final_path"""
     if os.path.dirname(os.path.normpath(final_path)) != "/dev":
         os.rename(tmp_path, final_path)
 
 
 @contextmanager
-def AtomicFileCreate(final_path: str, keep: bool = False) -> Iterator[str]:
+def AtomicFileCreate(final_path: StrPath, keep: bool = False) -> Iterator[str]:
     """Context manager to create a temporary file.  Entering returns path to
     the temporary file in the same directory as finalPath.  If the code in
-    context succeeds, the file renamed to its actually name.  If an error
+    context succeeds, the file renamed to its actual name.  If an error
     occurs, the file is not installed and is removed unless keep is specified.
     """
     tmp_path = atomic_tmp_file(final_path)
@@ -229,23 +262,23 @@ def make_public_dir(in_directory: str, suggested_name: Optional[str] = None) ->
     our old default.
     """
     if suggested_name is not None:
-        generated_dir_path: str = os.path.join(in_directory, suggested_name)
+        generated_dir_path1 = os.path.join(in_directory, suggested_name)
         try:
-            os.mkdir(generated_dir_path)
-            os.chmod(generated_dir_path, 0o777)
-            return generated_dir_path
+            os.mkdir(generated_dir_path1)
+            os.chmod(generated_dir_path1, 0o777)
+            return generated_dir_path1
         except FileExistsError:
             pass
     for i in range(
         4, 32 + 1
     ):  # make random uuids and truncate to lengths starting at 4 and working up to max 32
         for _ in range(10):  # make 10 attempts for each length
-            truncated_uuid: str = str(uuid.uuid4()).replace("-", "")[:i]
-            generated_dir_path: str = os.path.join(in_directory, truncated_uuid)
+            truncated_uuid = str(uuid.uuid4()).replace("-", "")[:i]
+            generated_dir_path2 = os.path.join(in_directory, truncated_uuid)
             try:
-                os.mkdir(generated_dir_path)
-                os.chmod(generated_dir_path, 0o777)
-                return generated_dir_path
+                os.mkdir(generated_dir_path2)
+                os.chmod(generated_dir_path2, 0o777)
+                return generated_dir_path2
             except FileExistsError:
                 pass
     this_should_never_happen: str = os.path.join(in_directory, str(uuid.uuid4()))
@@ -307,7 +340,7 @@ class WriteWatchingStream:
 
         self.backingStream = backingStream
         # We have no write listeners yet
-        self.writeListeners = []
+        self.writeListeners: list[Callable[[int], None]] = []
 
     def onWrite(self, listener: Callable[[int], None]) -> None:
         """
@@ -318,7 +351,7 @@ class WriteWatchingStream:
 
     # Implement the file API from https://docs.python.org/2.4/lib/bltin-file-objects.html
 
-    def write(self, data):
+    def write(self, data: bytes) -> None:
         """
         Write the given data to the file.
         """
@@ -330,7 +363,7 @@ class WriteWatchingStream:
             # Send out notifications
             listener(len(data))
 
-    def writelines(self, datas):
+    def writelines(self, datas: Iterable[bytes]) -> None:
         """
         Write each string from the given iterable, without newlines.
         """
@@ -338,14 +371,14 @@ class WriteWatchingStream:
         for data in datas:
             self.write(data)
 
-    def flush(self):
+    def flush(self) -> None:
         """
         Flush the backing stream.
         """
 
         self.backingStream.flush()
 
-    def close(self):
+    def close(self) -> None:
         """
         Close the backing stream.
         """
