@@ -13,6 +13,8 @@ from unittest.mock import patch
 from uuid import uuid4
 
 import pytest
+from pytest_httpserver import HTTPServer
+
 import WDL.Error
 import WDL.Expr
 
@@ -23,6 +25,7 @@ from toil.test import (
     needs_docker,
     needs_docker_cuda,
     needs_google_storage,
+    needs_online,
     needs_singularity_or_docker,
     needs_wdl,
     slow,
@@ -32,7 +35,6 @@ from toil.wdl.wdltoil import (
     WDLSectionJob,
     WDLWorkflowGraph,
     parse_disks,
-    remove_common_leading_whitespace,
 )
 
 logger = logging.getLogger(__name__)
@@ -113,11 +115,11 @@ class TestWDLConformance:
 
     @slow
     def test_unit_tests_v11(self, wdl_conformance_test_repo: Path) -> None:
-        # There are still some bugs with the WDL spec, use a fixed version until
-        # See comments of https://github.com/openwdl/wdl/pull/669
+        # TODO: Using a branch lets Toil commits that formerly passed start to
+        # fail CI when the branch moves.
         os.chdir(wdl_conformance_test_repo)
-        repo_url = "https://github.com/stxue1/wdl.git"
-        repo_branch = "wdl-1.1.3-fixes"
+        repo_url = "https://github.com/openwdl/wdl.git"
+        repo_branch = "wdl-1.1"
         commands1 = [
             exactPython,
             "setup_unit_tests.py",
@@ -249,6 +251,7 @@ class TestWDL:
                 assert os.path.exists(result["ga4ghMd5.value"])
                 assert os.path.basename(result["ga4ghMd5.value"]) == "md5sum.txt"
 
+    @needs_online
     def test_url_to_file(self, tmp_path: Path) -> None:
         """
         Test if web URL strings can be coerced to usable Files.
@@ -308,6 +311,59 @@ class TestWDL:
             assert isinstance(result["wait.result"], str)
             assert result["wait.result"] == "waited"
 
+    def test_restart(self, tmp_path: Path) -> None:
+        """
+        Test if a WDL workflow can be restarted and finish successfully.
+        """
+        with get_data("test/wdl/testfiles/read_file.wdl") as wdl:
+            out_dir = tmp_path / "out"
+            file_path = tmp_path / "file"
+            jobstore_path = tmp_path / "tree"
+            command = (
+                self.base_command
+                + [
+                    str(wdl),
+                    "-o",
+                    str(out_dir),
+                    "-i",
+                    json.dumps({"read_file.input_string": str(file_path)}),
+                    "--jobStore",
+                    str(jobstore_path),
+                    "--retryCount=0"
+                ]
+            )
+            with pytest.raises(subprocess.CalledProcessError):
+                # The first time we run it, it should fail because it's trying
+                # to work on a nonexistent file from a string path.
+                result_json = subprocess.check_output(
+                    command + ["--logCritical"]
+                )
+
+            # Then create the file
+            with open(file_path, "w") as f:
+                f.write("This is a line\n")
+                f.write("This is a different line")
+            
+            # Now it should work
+            result_json = subprocess.check_output(
+                    command + ["--restart"]
+                )
+            result = json.loads(result_json)
+
+            assert "read_file.lines" in result
+            assert isinstance(result["read_file.lines"], list)
+            assert result["read_file.lines"] == [
+                "This is a line",
+                "This is a different line"
+            ]
+
+            # Since we were catching
+            # <https://github.com/DataBiosphere/toil/issues/5247> at file
+            # export, make sure we actually exported a file.
+            assert "read_file.remade_file" in result
+            assert isinstance(result["read_file.remade_file"], str)
+            assert os.path.exists(result["read_file.remade_file"])
+
     @needs_singularity_or_docker
     def test_workflow_file_deletion(self, tmp_path: Path) -> None:
         """
@@ -597,7 +653,7 @@ class TestWDL:
                 != result_not_cached["random.value_written"]
             )
 
-    def test_url_to_optional_file(self, tmp_path: Path) -> None:
+    def test_url_to_optional_file(self, tmp_path: Path, httpserver: HTTPServer) -> None:
         """
         Test if missing and error-producing URLs are handled correctly for optional File? values.
         """
@@ -610,7 +666,15 @@ class TestWDL:
                 Return the parsed output.
                 """
                 logger.info("Test optional file with HTTP code %s", code)
-                json_value = '{"url_to_optional_file.http_code": %d}' % code
+                httpserver.expect_request(
+                    "/" + str(code)
+                ).respond_with_data(
+                    "Some data",
+                    status=code,
+                    content_type="text/plain"
+                )
+                base_url = httpserver.url_for("/")
+                json_value = '{"url_to_optional_file.http_code": %d, "url_to_optional_file.base_url": "%s"}' % (code, base_url)
                 result_json = subprocess.check_output(
                     self.base_command
                     + [
@@ -1055,7 +1119,7 @@ class TestWDLToilBench(unittest.TestCase):
                     assert "decl2" in result[0]
                     assert "successor" in result[1]
 
-    def make_string_expr(self, to_parse: str) -> WDL.Expr.String:
+    def make_string_expr(self, to_parse: str, expr_type: type[WDL.Expr.String] = WDL.Expr.String) -> WDL.Expr.String:
         """
         Parse pseudo-WDL for testing whitespace removal.
         """
@@ -1066,122 +1130,7 @@ class TestWDLToilBench(unittest.TestCase):
         for i in range(1, len(parts), 2):
             parts[i] = WDL.Expr.Placeholder(pos, {}, WDL.Expr.Null(pos))
 
-        return WDL.Expr.String(pos, parts)
-
-    def test_remove_common_leading_whitespace(self) -> None:
-        """
-        Make sure leading whitespace removal works properly.
-        """
-
-        # For a single line, we remove its leading whitespace
-        expr = self.make_string_expr(" a ~{b} c")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "a "
-        assert trimmed.parts[2] == " c"
-
-        # Whitespace removed isn't affected by totally blank lines
-        expr = self.make_string_expr("    \n\n    a\n    ~{stuff}\n    b\n\n")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "\n\na\n"
-        assert trimmed.parts[2] == "\nb\n\n"
-
-        # Unless blank toleration is off
-        expr = self.make_string_expr("    \n\n    a\n    ~{stuff}\n    b\n\n")
-        trimmed = remove_common_leading_whitespace(expr, tolerate_blanks=False)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "    \n\n    a\n    "
-        assert trimmed.parts[2] == "\n    b\n\n"
-
-        # Whitespace is still removed if the first line doesn't have it before the newline
-        expr = self.make_string_expr("\n    a\n    ~{stuff}\n    b\n")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "\na\n"
-        assert trimmed.parts[2] == "\nb\n"
-
-        # Whitespace is not removed if actual content is dedented
-        expr = self.make_string_expr("    \n\n    a\n    ~{stuff}\nuhoh\n    b\n\n")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "    \n\n    a\n    "
-        assert trimmed.parts[2] == "\nuhoh\n    b\n\n"
-
-        # Unless dedents are tolerated
-        expr = self.make_string_expr("    \n\n    a\n    ~{stuff}\nuhoh\n    b\n\n")
-        trimmed = remove_common_leading_whitespace(expr, tolerate_dedents=True)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "\n\na\n"
-        assert trimmed.parts[2] == "\nuhoh\nb\n\n"
-
-        # Whitespace is still removed if all-whitespace lines have less of it
-        expr = self.make_string_expr("\n    a\n    ~{stuff}\n  \n    b\n")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "\na\n"
-        assert trimmed.parts[2] == "\n\nb\n"
-
-        # Unless all-whitespace lines are not tolerated
-        expr = self.make_string_expr("\n    a\n    ~{stuff}\n  \n    b\n")
-        trimmed = remove_common_leading_whitespace(expr, tolerate_all_whitespace=False)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "\n  a\n  "
-        assert trimmed.parts[2] == "\n\n  b\n"
-
-        # When mixed tabs and spaces are detected, nothing is changed.
-        expr = self.make_string_expr("\n    a\n\t~{stuff}\n    b\n")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "\n    a\n\t"
-        assert trimmed.parts[2] == "\n    b\n"
-
-        # When mixed tabs and spaces are not in the prefix, whitespace is removed.
-        expr = self.make_string_expr("\n\ta\n\t~{stuff} \n\tb\n")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == "\na\n"
-        assert trimmed.parts[2] == " \nb\n"
-
-        # An empty string works
-        expr = self.make_string_expr("")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 1
-        assert trimmed.parts[0] == ""
-
-        # A string of only whitespace is preserved as an all-whitespece line
-        expr = self.make_string_expr("\t\t\t")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 1
-        assert trimmed.parts[0] == "\t\t\t"
-
-        # A string of only whitespace is trimmed when all-whitespace lines are not tolerated
-        expr = self.make_string_expr("\t\t\t")
-        trimmed = remove_common_leading_whitespace(expr, tolerate_all_whitespace=False)
-        assert len(trimmed.parts) == 1
-        assert trimmed.parts[0] == ""
-
-        # An empty expression works
-        expr = WDL.Expr.String(
-            WDL.Error.SourcePosition("nowhere", "nowhere", 0, 0, 0, 0), []
-        )
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 0
-
-        # An expression of only placeholders works
-        expr = self.make_string_expr("~{AAA}")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert len(trimmed.parts) == 3
-        assert trimmed.parts[0] == ""
-        assert trimmed.parts[2] == ""
-
-        # The command flag is preserved
-        expr = self.make_string_expr(" a ~{b} c")
-        trimmed = remove_common_leading_whitespace(expr)
-        assert trimmed.command == False
-        expr.command = True
-        trimmed = remove_common_leading_whitespace(expr)
-        assert trimmed.command == True
+        return expr_type(pos, parts)
 
     def test_choose_human_readable_directory(self) -> None:
         """
@@ -1195,7 +1144,7 @@ class TestWDLToilBench(unittest.TestCase):
 
         state: DirectoryNamingStateDict = {}
 
-        # The first time we should get  apath with the task name and without the ID
+        # The first time we should get a path with the task name and without the ID
         first_chosen = choose_human_readable_directory(
             "root", "taskname", "111-222-333", state
         )
